<?php

/**
 * \file Anomalies.class.inc
 * \class Anomalies
 *
 * \author Ludivine Premont <ludivine.premont@veremes.com>.
 *
 *   \brief This file contains the Modes php class
 *
 * This class defines Rest Api to Vitis Anomalies
 *
 */
require_once 'AnomaliesAnomalies.class.inc';
require_once dirname($_SERVER['SCRIPT_FILENAME']) . '/class/vitis_lib/Connection.class.inc';
require_once 'Anomalie.class.inc';
require_once dirname($_SERVER['SCRIPT_FILENAME']) . '/class/vmlib/BdDataAccess.inc';
require_once dirname($_SERVER['SCRIPT_FILENAME']) . '/class/vmlib/Email.class.inc';

class Anomalies extends AnomaliesAnomalies {
  /**
   * @SWG\Definition(
   *   definition="/anomalie",
   *   allOf={
   *     @SWG\Schema(ref="#/definitions/anomalie")
   *   }
   * )
   * * @SWG\Tag(
   *   name="Anomalies",
   *   description="Operations about Anomalies"
   * )
   */

  /**
   * construct
   * @param type $aPath url of the request
   * @param type $aValues parameters of the request
   * @param type $properties properties
   * @param type $bShortcut false to reinit variables
   * @param type $oConnection connection object
   */
  function __construct($aPath, $aValues, $properties, $bShortcut = false, $oConnection = false) {
    parent::__construct($aPath, $aValues, $properties, $bShortcut, $oConnection);
    $this->aSelectedFields = Array('anomalies_id', 'status_id', 'status_name', 'status_color', 'theme_id', 'theme_name', 'theme_short_name', 'title', 'description', 'files', 'contact', 'user_id', 'login', 'user_name', 'email', 'anomalies_date', 'last_update_date', 'closing_date', 'map_id', 'admin_comment', 'geom');
  }

  /**
   * @SWG\Get(path="/anomalies",
   *   tags={"Anomalies"},
   *   summary="Get Anomalies",
   *   description="Request to get Anomalies",
   *   operationId="GET",
   *   produces={"application/xml", "application/json", "application/x-vm-json"},
   *   @SWG\Parameter(
   *     name="token",
   *     in="query",
   *     description="user token",
   *     required=true,
   *     type="string"
   *   ),
   * @SWG\Parameter(
   *     name="order_by",
   *     in="query",
   *     description="list of ordering fields",
   *     required=false,
   *     type="string"
   *   ),
   * @SWG\Parameter(
   *     name="sort_order",
   *     in="query",
   *     description="sort order",
   *     required=false,
   *     type="string"
   *   ),
   * @SWG\Parameter(
   *     name="limit",
   *     in="query",
   *     description="number of element",
   *     required=false,
   *     type="integer",
   *     format="int32"
   *   ),
   * @SWG\Parameter(
   *     name="offset",
   *     in="query",
   *     description="index of first element",
   *     required=false,
   *     type="string",
   *     format="int32"
   *   ),
   * @SWG\Parameter(
   *     name="attributs",
   *     in="query",
   *     description="list of attributs",
   *     required=false,
   *     type="string"
   *   ),
   * @SWG\Parameter(
   *     name="filter",
   *     in="query",
   *     description="filter results",
   *     required=false,
   *     type="string"
   *   ),
   * @SWG\Parameter(
   *     name="distinct",
   *     in="query",
   *     description="delete duplicates",
   *     required=false,
   *     type="boolean"
   *   ),
   *   @SWG\Response(
   *         response=200,
   *         description="Properties Response",
   *         @SWG\Schema(ref="#/definitions/anomalies")
   *     )
   *  )
   */

  /**
   * get Anomalies
   * @return  Anomalies
   */
  function GET() {
    $aReturn = $this->genericGet($this->aProperties['schema_anomalies'], "v_anomalies", "anomalies_id", false, 'anomalies', 'anomalies');
    return $aReturn['sMessage'];
  }

  /**
   * @SWG\Post(path="/anomalies",
   *   tags={"Anomalies"},
   *   summary="Add Anomaly",
   *   description="Request to add anomaly",
   *   operationId="POST",
   *   produces={"application/xml", "application/json"},
   *   @SWG\Parameter(
   *     name="token",
   *     in="formData",
   *     description="user token",
   *     required=true,
   *     type="string"
   *   ),
   *   @SWG\Parameter(
   *     name="geom",
   *     in="formData",
   *     description="geom of the anomaly",
   *     required=true,
   *     type="string"
   *   ),
   *   @SWG\Parameter(
   *     name="theme_id",
   *     in="formData",
   *     description="theme of the anomaly",
   *     required=true,
   *     type="integer"
   *   ),
   *   @SWG\Parameter(
   *     name="title",
   *     in="formData",
   *     description="title of the anomaly",
   *     required=true,
   *     type="string"
   *   ),
   *   @SWG\Parameter(
   *     name="description",
   *     in="formData",
   *     description="description of the anomaly",
   *     required=true,
   *     type="string"
   *   ),
   *   @SWG\Parameter(
   *     name="files",
   *     in="formData",
   *     description="files of the anomaly",
   *     required=false,
   *     type="string"
   *   ),
   *   @SWG\Parameter(
   *     name="contact",
   *     in="formData",
   *     description="contact who declare the anomaly",
   *     required=true,
   *     type="string"
   *   ),
   *   @SWG\Parameter(
   *     name="user_id",
   *     in="formData",
   *     description="user who declare the anomaly",
   *     required=true,
   *     type="integer"
   *   ),
   *   @SWG\Parameter(
   *     name="map_id",
   *     in="formData",
   *     description="map of the anomaly",
   *     required=true,
   *     type="integer"
   *   ),
   *   @SWG\Response(
   *         response=200,
   *         description="Poprerties Response",
   *         @SWG\Schema(ref="#/definitions/anomalies")
   *     )
   *  )
   */

  /**
   * insert anomaly
   * @return id of the anomaly created
   */
  function POST() {
    require 'Anomalies.class.sql.inc';

    // Selectionne le status par défaut
    $oBd = new BD($this->aProperties["db_superuser_login"], $this->aProperties["db_superuser_pass"], $this->aProperties["database"], $this->aProperties["server"], $this->aProperties["port"], $this->aProperties["sgbd"]);
    $aParams = array(
      "SCHEMA" => array("value" => $this->aProperties["schema_anomalies"], "type" => "column_name"),
      "STATUS_NAME" => array("value" => 'Nouveau', "type" => "string"),
    );
    $oPDOresult = $oBd->executeWithParams($aSql['selectStatus'], $aParams);
    $oStatus = $oBd->ligneSuivante($oPDOresult);

    $this->aValues['status_id'] = $oStatus['status_id'];

    // Si il a au moins un fichier
    if (!empty($_FILES['files'])) {
  
      // On vérifie que la somme des tailles des fichiers est plus petite que 20Mo
      if ($this->getFilesSize() < (20 * 1000000)) {
        $aReturn = $this->genericPost($this->aProperties['schema_anomalies'], 'anomalies', $this->aProperties['schema_anomalies'] . '.seq_anomalies_id', 'anomalies_id', array('*'), 'anomalies', 'anomalies');
      } else {
        // ERROR
        $aReturn = array();
        $aReturn['status'] = 0;
        $aReturn['errorMessage'] = "Les fichiers sont trop volumineux";
        return json_encode($aReturn);
      }
    } else {
      $aReturn = $this->genericPost($this->aProperties['schema_anomalies'], 'anomalies', $this->aProperties['schema_anomalies'] . '.seq_anomalies_id', 'anomalies_id');
    }

    // Envoi d'une email de confirmation de création a l'utilisateur (si il a une adresse) et à l'admin
    $oMessage = json_decode($aReturn['sMessage']);
    if ($oMessage->status == 1) {
      $oAnomaly = $this->getAnomaly($oMessage->anomalies_id);
      $this->prepareAndSendEmail('email_user_create_anomaly', $oAnomaly, 'email_object_user_create_anomaly', false);
      $this->prepareAndSendEmail('email_admin_create_anomaly', $oAnomaly, 'email_object_admin_create_anomaly', true);
    }

    return $aReturn['sMessage'];
  }

  /**
   * @SWG\Post(path="/anomalies/{anomalies_id}",
   *   tags={"Anomalies"},
   *   summary="Add Anomaly",
   *   description="Request to add anomaly",
   *   operationId="POST",
   *   produces={"application/xml", "application/json"},
   *   @SWG\Parameter(
   *     name="token",
   *     in="formData",
   *     description="user token",
   *     required=true,
   *     type="string"
   *   ),
   *   @SWG\Parameter(
   *     name="geom",
   *     in="formData",
   *     description="geom of the anomaly",
   *     required=true,
   *     type="string"
   *   ),
   * * @SWG\Parameter(
   *     name="theme_id",
   *     in="formData",
   *     description="theme of the anomaly",
   *     required=true,
   *     type="integer"
   *   ),
   *   @SWG\Parameter(
   *     name="title",
   *     in="formData",
   *     description="title of the anomaly",
   *     required=true,
   *     type="string"
   *   ),
   *   @SWG\Parameter(
   *     name="description",
   *     in="formData",
   *     description="description of the anomaly",
   *     required=true,
   *     type="string"
   *   ),
   *   @SWG\Parameter(
   *     name="files",
   *     in="formData",
   *     description="files of the anomaly",
   *     required=false,
   *     type="string"
   *   ),
   *   @SWG\Parameter(
   *     name="contact",
   *     in="formData",
   *     description="contact who declare the anomaly",
   *     required=true,
   *     type="string"
   *   ),
   *   @SWG\Parameter(
   *     name="status_id",
   *     in="formData",
   *     description="status of the anomaly",
   *     required=true,
   *     type="integer"
   *   ),
   *   @SWG\Parameter(
   *     name="admin_comment",
   *     in="formData",
   *     description="map of the anomaly",
   *     required=true,
   *     type="string"
   *   ),
   *   @SWG\Parameter(
   *     name="geom",
   *     in="formData",
   *     description="geom of the anomaly",
   *     required=true,
   *     type="string"
   *   ),
   *   @SWG\Response(
   *         response=200,
   *         description="Poprerties Response",
   *         @SWG\Schema(ref="#/definitions/anomalies")
   *     )
   *  )
   */

  /**
   * update anomaly
   * @return id of the anomaly created
   */
  function PUT() {

    $oAnomaly = $this->getAnomaly($this->aValues['anomalies_id']);
    $this->aValues['last_update_date'] = 'NOW()';
    // Le statut vaut 'terminé' (4) ou 'annulé' (5), initialise la date de clôture
    if ($this->aValues['status_id'] == 4 || $this->aValues['status_id'] == 5) {
      $this->aValues['closing_date'] = 'NOW()';
    }

    if ($this->aValues['status_id'] == 4 && (!isset($this->aValues['admin_comment']) || $this->aValues['admin_comment'] === '')) {
      // ERROR
      $aReturn = array();
      $aReturn['status'] = 0;
      $aReturn['errorMessage'] = "Le commentaire administrateur est obligatoire pour clôturer une demande";
      return json_encode($aReturn);
    }

    // Si il a au moins un fichier
    if (!empty($_FILES['files'])) {
      // On vérifie que la somme des tailles des fichiers est plus petite que 20Mo
      if ($this->getFilesSize() < (20 * 1000000)) {
        $aReturn = $this->genericPut($this->aProperties['schema_anomalies'], 'anomalies', 'anomalies_id', array('*'), 'anomalies', 'anomalies');
      } else {
        // ERROR
        $aReturn = array();
        $aReturn['status'] = 0;
        $aReturn['errorMessage'] = "Les fichiers sont trop volumineux";
        return json_encode($aReturn);
      }
    } else if (!empty($this->aValues['vitis_deleted_files'])) {
      $aReturn = $this->genericPut($this->aProperties['schema_anomalies'], 'anomalies', 'anomalies_id', array('*'), 'anomalies', 'anomalies');
    } else {
      $aReturn = $this->genericPut($this->aProperties['schema_anomalies'], 'anomalies', 'anomalies_id');
    }

    // Envoi d'une email de confirmation de création a l'utilisateur (si il a une adresse) et à l'admin
    $oMessage = json_decode($aReturn['sMessage']);
    if ($oMessage->status == 1) {
      $emailUserTitle = '';
      $emailAdminTitle = '';
      $emailUserId = '';
      $emailAdminId = '';

      // Si le status change
      if (isset($this->aValues['status_id']) && $oAnomaly['status_id'] != $this->aValues['status_id']) {
        $emailAdminTitle = 'email_object_admin_status_change_anomaly';
        $emailAdminId = 'email_admin_status_change_anomaly';
        switch ($this->aValues['status_id']) {
          case '2': // En attente d’information
            $emailUserTitle = 'email_object_user_status_waiting_anomaly';
            $emailUserId = 'email_user_status_waiting_anomaly';
            break;
          case '3': // En cours
            $emailUserTitle = 'email_object_user_status_in_progress_anomaly';
            $emailUserId = 'email_user_status_in_progress_anomaly';
            break;
          case '4': // Terminé
            $emailUserTitle = 'email_object_user_status_finished_anomaly';
            $emailUserId = 'email_user_status_finished_anomaly';
            $emailAdminTitle = 'email_object_admin_finished_anomaly';
            $emailAdminId = 'email_admin_finished_anomaly';
            break;
          case '5': // Annulé
            $emailUserTitle = 'email_object_user_status_cancel_anomaly';
            $emailUserId = 'email_user_status_cancel_anomaly';
            break;
          default:
            break;
        }
      } else if ($oAnomaly['login'] == $_SESSION["ses_Login"]) { // Si la modification vient de l'utilisateur
        $emailUserTitle = 'email_object_user_update_anomaly';
        $emailAdminTitle = 'email_object_admin_update_anomaly';
        $emailUserId = 'email_user_update_anomaly';
        $emailAdminId = 'email_admin_update_anomaly';
      }

      $oAnomaly = $this->getAnomaly($oMessage->anomalies_id);
      if ($emailUserTitle != '' && $emailUserId != '') {
        $this->prepareAndSendEmail($emailUserId, $oAnomaly, $emailUserTitle, false);
      }
      if ($emailAdminTitle != '' && $emailAdminId != '') {
        $this->prepareAndSendEmail($emailAdminId, $oAnomaly, $emailAdminTitle, true);
      }
    }

    return $aReturn['sMessage'];
  }

  function prepareAndSendEmail($propertiesId, $oAnomaly, $sTitleId, $isAdminMail) {

    $sTemplateEmail = $this->getTemplateEmail($propertiesId, $oAnomaly);
    $sTitle = $this->getTemplateEmail($sTitleId, $oAnomaly);

    $this->sendMail($oAnomaly, $sTemplateEmail, $sTitle, $isAdminMail);
  }

  function getAnomaly($anomalyId) {
    require 'Anomalies.class.sql.inc';

    // Selectionne les informations sur l'anomalie
    $oBd = new BD($this->aProperties["db_superuser_login"], $this->aProperties["db_superuser_pass"], $this->aProperties["database"], $this->aProperties["server"], $this->aProperties["port"], $this->aProperties["sgbd"], 'UTF8');
    $aParams = array(
      "SCHEMA" => array("value" => $this->aProperties["schema_anomalies"], "type" => "column_name"),
      "ANOMALY_ID" => array("value" => $anomalyId, "type" => "number"),
    );

    $oPDOresult = $oBd->executeWithParams($aSql['selectAnomaly'], $aParams);
    $oAnomaly = $oBd->ligneSuivante($oPDOresult);
    return $oAnomaly;
  }
  
  function getTemplateEmail($propertiesId, $oAnomaly) {
    $sTemplateEmail = $this->aProperties[$propertiesId];

    // Remplace 
    foreach ($oAnomaly as $key => $value) {
      if ($key === 'anomalies_date' || $key == 'last_update_date' || $key == 'closing_date' && $value != '') {
        try {
          $date = new \DateTime($value);
          $sTemplateEmail = str_replace('{{anomaly.'.$key.'}}', $date->format($this->aProperties['date_format']), $sTemplateEmail);
        } catch (\Exception $e) {
          $sTemplateEmail = str_replace('{{anomaly.'.$key.'}}', $value, $sTemplateEmail);
        }
      } else {
        $sTemplateEmail = str_replace('{{anomaly.' . $key . '}}', $value, $sTemplateEmail);
      }
    }

    return $sTemplateEmail;
  }

  function sendMail($oAnomaly, $sBody, $sTitle, $isAdminMail) {

    if ($isAdminMail) {
      $sTo = $this->aProperties['anomalies_email_admin'];
    } else if (isset($oAnomaly['user_email']) && $oAnomaly['user_email'] !== '') {
      $sTo = $oAnomaly['user_email'];
    }

    if (isset($sTo) && $sTo !== '') {
      $sMailStruct = array(
          "to" => $sTo,
          "cci"=> "",
          "subject" => $this->aProperties['mail_tag_subject'] . $sTitle,
          "body" => $sBody,
          "aAttachments" => array()
      );
      $mail = new Email(null, null, $this->aProperties, null);
      $mail->loadFromExernalDataStruct($sMailStruct);
      $mail->send();
    }
  }

  /**
   * Récupère la taille des fichiers existant + la taille des fichiers télédéchargé
   * 
   * @return {integer}
   */
  function getFilesSize() {
    $size = 0;

    // Si il y a des fichiers que l'on garde, on ajoute aussi leur poids
    $sFolder = $this->aProperties['ws_data_dir'] . '/anomalies/anomalies/documents/' . $this->aValues['my_vitis_id'] . '/files';
    if (isset($this->aValues['vitis_unchanged_files'])) {
      $aUnchangedFiles = json_decode($this->aValues['vitis_unchanged_files'], true);
      if (is_array($aUnchangedFiles['files']) && is_dir($sFolder)) {
        $aFolder = scandir($sFolder);
        foreach ($aFolder as $sFile) {
          if ($sFile != "." && $sFile != "..") {
            if (!is_dir($sFolder . "/" . $sFile) && in_array($sFile, $aUnchangedFiles['files'])) {
              $size = $size +  filesize($sFolder . "/" . $sFile);
            }
          }
        }
      }
    }

    // Ajoute le poids des fichiers ajouté
    if ($_FILES['files']['size']) {
      $size = $_FILES['files']['size'];
    } else {
      for ($i=0; $i < count($_FILES['files']); $i++) {
        $size = $size + $_FILES['files'][$i]['size'];
      }
    }

    return $size;
  }
}
?>