<?php

/**
 * \file AnomaliesFeatureStyles.class.inc
 * \class AnomaliesFeatureStyles
 *
 * \author Ludivine Premont <ludivine.premont@veremes.com>.
 *
 *   \brief This file contains the Modes php class
 *
 * This class defines Rest Api to Vitis AnomaliesFeatureStyles
 *
 */
require_once 'AnomaliesAnomalies.class.inc';
require_once dirname($_SERVER['SCRIPT_FILENAME']) . '/class/vitis_lib/Connection.class.inc';
require_once 'AnomaliesFeatureStyle.class.inc';
require_once dirname($_SERVER['SCRIPT_FILENAME']) . '/class/vmlib/BdDataAccess.inc';

class AnomaliesFeatureStyles extends AnomaliesAnomalies {
  /**
   * @SWG\Definition(
   *   definition="/anomaliesfeaturestyle",
   *   allOf={
   *     @SWG\Schema(ref="#/definitions/anomaliesfeaturestyle")
   *   }
   * )
   * * @SWG\Tag(
   *   name="AnomaliesFeatureStyles",
   *   description="Operations about AnomaliesFeatureStyles"
   * )
   */

  /**
   * construct
   * @param type $aPath url of the request
   * @param type $aValues parameters of the request
   * @param type $properties properties
   * @param type $bShortcut false to reinit variables
   * @param type $oConnection connection object
   */
  function __construct($aPath, $aValues, $properties, $bShortcut = false, $oConnection = false) {
    parent::__construct($aPath, $aValues, $properties, $bShortcut, $oConnection);
    $this->aSelectedFields = Array(
      'anomalies_feature_style_id',
      'theme_name',
      'status_name',
      'feature_style_id',
      'draw_color',
      'draw_outline_color',
      'draw_size',
      'draw_dash',
      'draw_symbol',
      'draw_rotation',
      'image',
      'text_font',
      'text_color',
      'text_outline_color',
      'text_size',
      'text_outline_size',
      'text_offset_x',
      'text_offset_y',
      'text_rotation',
      'text_text'
    );
  }

  /**
   * @SWG\Get(path="/anomaliesfeaturestyles",
   *   tags={"AnomaliesFeatureStyles"},
   *   summary="Get AnomaliesFeatureStyles",
   *   description="Request to get AnomaliesFeatureStyles",
   *   operationId="GET",
   *   produces={"application/xml", "application/json", "application/x-vm-json"},
   *   @SWG\Parameter(
   *     name="token",
   *     in="query",
   *     description="user token",
   *     required=true,
   *     type="string"
   *   ),
   * @SWG\Parameter(
   *     name="order_by",
   *     in="query",
   *     description="list of ordering fields",
   *     required=false,
   *     type="string"
   *   ),
   * @SWG\Parameter(
   *     name="sort_order",
   *     in="query",
   *     description="sort order",
   *     required=false,
   *     type="string"
   *   ),
   * @SWG\Parameter(
   *     name="limit",
   *     in="query",
   *     description="number of element",
   *     required=false,
   *     type="integer",
   *     format="int32"
   *   ),
   * @SWG\Parameter(
   *     name="offset",
   *     in="query",
   *     description="index of first element",
   *     required=false,
   *     type="string",
   *     format="int32"
   *   ),
   * @SWG\Parameter(
   *     name="attributs",
   *     in="query",
   *     description="list of attributs",
   *     required=false,
   *     type="string"
   *   ),
   * @SWG\Parameter(
   *     name="filter",
   *     in="query",
   *     description="filter results",
   *     required=false,
   *     type="string"
   *   ),
   * @SWG\Parameter(
   *     name="distinct",
   *     in="query",
   *     description="delete duplicates",
   *     required=false,
   *     type="boolean"
   *   ),
   *   @SWG\Response(
   *         response=200,
   *         description="Properties Response",
   *         @SWG\Schema(ref="#/definitions/anomaliesfeaturestyles")
   *     )
   *  )
   */

  /**
   * get Anomalies
   * @return  Anomalies
   */
  function GET() {
    $aReturn = $this->genericGet($this->aProperties['schema_anomalies'], "v_anomalies_feature_style", "anomalies_feature_style_id");

    for ($i = 0; $i < count($this->aObjects); $i++) {
      $this->aObjects[$i]->aFields['style'] = $this->getGeoJSONStyle($this->aObjects[$i]->aFields);
    }
    $aXmlRacineAttribute['status'] = 1;
    $sMessage = $this->asDocument('', 'vitis', $this->aValues['sEncoding'], True, $aXmlRacineAttribute, $this->aValues['sSourceEncoding'], $this->aValues['output']);
    return $sMessage;
  }

  /**
   * Returns a GeoJSON format style from an array format style
   * @param array $aStyle
   * @return array
   */
  function getGeoJSONStyle($aStyle) {
    $aGeoJSONStyle = array(
      'draw' => array(),
      'text' => array(),
    );
    // Draw
    if (!empty($aStyle['draw_color'])) {
      $aGeoJSONStyle['draw']['color'] = $aStyle['draw_color'];
    }
    if (!empty($aStyle['draw_outline_color'])) {
      $aGeoJSONStyle['draw']['outline_color'] = $aStyle['draw_outline_color'];
    }
    if (!empty($aStyle['draw_size'])) {
      $aGeoJSONStyle['draw']['size'] = $aStyle['draw_size'];
    }
    if (!empty($aStyle['draw_dash'])) {
      $aGeoJSONStyle['draw']['dash'] = $aStyle['draw_dash'];
    }
    if (!empty($aStyle['draw_symbol'])) {
      $aGeoJSONStyle['draw']['symbol'] = $aStyle['draw_symbol'];
    }
    if (!empty($aStyle['draw_rotation'])) {
      $aGeoJSONStyle['draw']['rotation'] = $aStyle['draw_rotation'];
    }
    if (!empty($aStyle['image'])) {
      $aGeoJSONStyle['draw']['image'] = $aStyle['image'];
    }
    // Text
    if (!empty($aStyle['text_font'])) {
      $aGeoJSONStyle['text']['font'] = $aStyle['text_font'];
    }
    if (!empty($aStyle['text_color'])) {
      $aGeoJSONStyle['text']['color'] = $aStyle['text_color'];
    }
    if (!empty($aStyle['text_outline_color'])) {
      $aGeoJSONStyle['text']['outline_color'] = $aStyle['text_outline_color'];
    }
    if (!empty($aStyle['text_size'])) {
      $aGeoJSONStyle['text']['size'] = $aStyle['text_size'];
    }
    if (!empty($aStyle['text_outline_size'])) {
      $aGeoJSONStyle['text']['outline_size'] = $aStyle['text_outline_size'];
    }
    if (!empty($aStyle['text_offset_x'])) {
      $aGeoJSONStyle['text']['offsetX'] = $aStyle['text_offset_x'];
    }
    if (!empty($aStyle['text_offset_y'])) {
      $aGeoJSONStyle['text']['offsetY'] = $aStyle['text_offset_y'];
    }
    if (!empty($aStyle['text_rotation'])) {
      $aGeoJSONStyle['text']['rotation'] = $aStyle['text_rotation'];
    }
    if (!empty($aStyle['text_text'])) {
      $aGeoJSONStyle['text']['text'] = $aStyle['text_text'];
    }
    // Vide draw ou text si ils sont vides
    if (empty($aGeoJSONStyle['draw'])) {
      unset($aGeoJSONStyle['draw']);
    }
    if (empty($aGeoJSONStyle['text'])) {
      unset($aGeoJSONStyle['text']);
    }
    return $aGeoJSONStyle;
  }
}
?>