<?php

/**
 * \file Themes.class.inc
 * \class Themes
 *
 * \author Ludivine Premont <ludivine.premont@veremes.com>.
 *
 *   \brief This file contains the Modes php class
 *
 * This class defines Rest Api to Vitis Themes
 *
 */
require_once 'AnomaliesAnomalies.class.inc';
require_once dirname($_SERVER['SCRIPT_FILENAME']) . '/class/vitis_lib/Connection.class.inc';
require_once 'Theme.class.inc';
require_once dirname($_SERVER['SCRIPT_FILENAME']) . '/class/vmlib/BdDataAccess.inc';

class Themes extends AnomaliesAnomalies {
  /**
   * @SWG\Definition(
   *   definition="/theme",
   *   allOf={
   *     @SWG\Schema(ref="#/definitions/theme")
   *   }
   * )
   * * @SWG\Tag(
   *   name="Themes",
   *   description="Operations about Themes"
   * )
   */

  /**
   * construct
   * @param type $aPath url of the request
   * @param type $aValues parameters of the request
   * @param type $properties properties
   * @param type $bShortcut false to reinit variables
   * @param type $oConnection connection object
   */
  function __construct($aPath, $aValues, $properties, $bShortcut = false, $oConnection = false) {
    parent::__construct($aPath, $aValues, $properties, $bShortcut, $oConnection);
    $this->aSelectedFields = Array('theme_id', 'name', 'short_name','description');
  }

  /**
   * @SWG\Get(path="/themes",
   *   tags={"Themes"},
   *   summary="Get Themes",
   *   description="Request to get Themes",
   *   operationId="GET",
   *   produces={"application/xml", "application/json", "application/x-vm-json"},
   *   @SWG\Parameter(
   *     name="token",
   *     in="query",
   *     description="user token",
   *     required=true,
   *     type="string"
   *   ),
   * @SWG\Parameter(
   *     name="order_by",
   *     in="query",
   *     description="list of ordering fields",
   *     required=false,
   *     type="string"
   *   ),
   * @SWG\Parameter(
   *     name="sort_order",
   *     in="query",
   *     description="sort order",
   *     required=false,
   *     type="string"
   *   ),
   * @SWG\Parameter(
   *     name="limit",
   *     in="query",
   *     description="number of element",
   *     required=false,
   *     type="integer",
   *     format="int32"
   *   ),
   * @SWG\Parameter(
   *     name="offset",
   *     in="query",
   *     description="index of first element",
   *     required=false,
   *     type="string",
   *     format="int32"
   *   ),
   * @SWG\Parameter(
   *     name="attributs",
   *     in="query",
   *     description="list of attributs",
   *     required=false,
   *     type="string"
   *   ),
   * @SWG\Parameter(
   *     name="filter",
   *     in="query",
   *     description="filter results",
   *     required=false,
   *     type="string"
   *   ),
   * @SWG\Parameter(
   *     name="distinct",
   *     in="query",
   *     description="delete duplicates",
   *     required=false,
   *     type="boolean"
   *   ),
   *   @SWG\Response(
   *         response=200,
   *         description="Properties Response",
   *         @SWG\Schema(ref="#/definitions/themes")
   *     )
   *  )
   */

  /**
   * get Themes
   * @return  Themes
   */
  function GET() {
    $aReturn = $this->genericGet($this->aProperties['schema_anomalies'], "anomalies_theme", "theme_id");
    return $aReturn['sMessage'];
  }
}
?>